#include "Token.h"

int Token::BI_RANGE_BEGIN = 0x040;
int Token::BI_RANGE_END = 0x0af;

int Token::II_RANGE_BEGIN = 0x0b0;
int Token::II_RANGE_END = 0x0ef;

int Token::BO_RANGE_BEGIN = 0x140;
int Token::BO_RANGE_END = 0x17f;

int Token::IO_RANGE_BEGIN = 0x180;
int Token::IO_RANGE_END = 0x1af;

int Token::CMD_RANGE_BEGIN = 0x0f0;
int Token::CMD_RANGE_END = 0x0fd;

bool Token::isEmpty() {
  return this->type == TokenType::EMPTY;
}

bool Token::isUndefined() {
  return this->type == TokenType::UNDEFINED;
}

bool Token::isBinaryInput() {
  return (int) this->type >= Token::BI_RANGE_BEGIN
         && (int) this->type <= Token::BI_RANGE_END;
}

bool Token::isIntegerInput() {
  return (int) this->type >= Token::II_RANGE_BEGIN
         && (int) this->type <= Token::II_RANGE_END;
}

bool Token::isNumericInput() {
  return this->type == TokenType::NI_MAJOR_VERSION
         || this->type == TokenType::NI_MINOR_VERSION;
}

bool Token::isBinaryOutput() {
  return (int) this->type >= Token::BO_RANGE_BEGIN
         && (int) this->type <= Token::BO_RANGE_END;
}

bool Token::isIntegerOutput() {
  return (int) this->type >= Token::IO_RANGE_BEGIN
         && (int) this->type <= Token::IO_RANGE_END;
}

bool Token::isCommand() {
  return (int) this->type >= Token::CMD_RANGE_BEGIN
         && (int) this->type <= Token::CMD_RANGE_END;
}

bool Token::isImmediate() {
  return this->type == TokenType::IMMEDIATE;
}

int Token::getBlockId(int _numBlocks) {
  int blockId = -1;
  if ((int) this->type >= (int) TokenType::CMD_BLOCK0
      && (int) this->type <= (int) TokenType::CMD_BLOCK0 + _numBlocks) {
    blockId = (int) this->type - (int) TokenType::CMD_BLOCK0;
  }
  return blockId;
}

